package javawrapper;

/**
* Controls the data relating to growth behaviors.
* <p>Copyright: Copyright (c) 2003 Charles D. Canham</p>
* <p>Company: Institute of Ecosystem Studies</p>
* @author Lora E. Murphy
* @version 1.0
*
* <br>Edit history:
* <br>------------------
* <br>Cleared the edit history at the end of 6.09
* <br>February 4, 2008: Updated to conform to Java 6 compliance (LEM)
* <br>April 11, 2008: Added the allometric diameter growth incrementer (LEM)
* <br>April 15, 2008: Added the height relative growth behavior (LEM)
*/

public class GrowthBehaviors
    extends BehaviorTypeBase {

  /**Asymptotic diameter growth for each species*/
  protected ModelVector mp_fAsymptoticDiameterGrowth = new ModelVector(
      "Asymptotic Diameter Growth (A)",
      "gr_asympDiameterGrowth", "gr_adgVal", 0, ModelVector.FLOAT);
  
  /**Asymptotic height growth for each species*/
  protected ModelVector mp_fAsymptoticHeightGrowth = new ModelVector(
      "Asymptotic Height Growth (A)",
      "gr_asympHeightGrowth", "gr_ahgVal", 0, ModelVector.FLOAT);

  /**Slope of growth response for each species*/
  protected ModelVector mp_fSlopeOfGrowthResponse = new ModelVector(
      "Slope of Diameter Growth Response (S)", "gr_slopeGrowthResponse",
      "gr_sgrVal", 0,  ModelVector.FLOAT);
  
  /**Slope of height growth response for each species*/
  protected ModelVector mp_fSlopeOfHeightGrowthResponse = new ModelVector(
      "Slope of Height Growth Response (S)", "gr_slopeHeightGrowthResponse",
      "gr_shgrVal", 0,  ModelVector.FLOAT);

  /**Diameter exponent for relative Michaelis-Menton growth*/
  protected ModelVector mp_fRelGrowthDiamExp = new ModelVector(
      "Relative Michaelis-Menton Growth - Diameter Exponent",
      "gr_relGrowthDiamExp", "gr_rgdeVal", 0, ModelVector.FLOAT);
  
  /**Height exponent for relative Michaelis-Menton growth*/
  protected ModelVector mp_fRelGrowthHeightExp = new ModelVector(
      "Relative Michaelis-Menton Growth - Height Exponent",
      "gr_relHeightGrowthHeightExp", "gr_rhgheVal", 0, ModelVector.FLOAT);

  /**Adult constant radial growth for each species, in mm/yr*/
  protected ModelVector mp_fAdultConstantRadialGrowth = new ModelVector(
      "Adult Constant Radial Growth in mm/yr", "gr_adultConstRadialInc",
      "gr_acriVal", 0, ModelVector.FLOAT);

  /**Adult constant area increment, in cm2/yr*/
  protected ModelVector mp_fAdultConstantAreaInc = new ModelVector(
      "Adult Constant Area Growth in sq. cm/yr", "gr_adultConstAreaInc",
      "gr_acaiVal", 0, ModelVector.FLOAT);

  /**NCI max potential growth for each species*/
  protected ModelVector mp_fNCIMaxPotentialGrowth = new ModelVector(
      "NCI Maximum Potential Growth, cm/yr",
      "gr_nciMaxPotentialGrowth",
      "gr_nmpgVal", 0,
      ModelVector.FLOAT);

  /**NCI maximum crowding distance, in m, for each species*/
  protected ModelVector mp_fNCIMaxCrowdingRadius = new ModelVector(
      "NCI Maximum Crowding Distance, in meters", "gr_nciMaxCrowdingRadius",
      "gr_nmcrVal", 0, ModelVector.FLOAT);

  /**NCI minimum DBH for crowding neighbors, for each species; all species
   * required*/
  protected ModelVector mp_fNCIMinNeighborDBH = new ModelVector(
      "NCI Minimum Neighbor DBH, in cm", "gr_nciMinNeighborDBH",
      "gr_nmndVal", 0, ModelVector.FLOAT, true);

  /**NCI alpha for each species*/
  protected ModelVector mp_fNCIAlpha = new ModelVector("NCI Alpha",
      "gr_nciAlpha", "gr_naVal", 0,
      ModelVector.FLOAT);

  /**NCI beta for each species*/
  protected ModelVector mp_fNCIBeta = new ModelVector("NCI Beta", "gr_nciBeta",
      "gr_nbVal", 0,
      ModelVector.FLOAT);

  /**NCI growth - Size sensitivity to NCI parameter (gamma)*/
  protected ModelVector mp_fNCISizeSensToNCI = new ModelVector(
      "NCI Size Sensitivity to NCI (gamma)", "gr_nciSizeSensNCI",
      "gr_nssnVal", 0, ModelVector.FLOAT);

  /**NCI growth - Crowding Effect Slope (C)*/
  protected ModelVector mp_fNCICrowdingEffectSlope = new ModelVector(
      "NCI Crowding Effect Slope (C)", "gr_nciCrowdingSlope", "gr_ncslVal", 0,
      ModelVector.FLOAT);

  /**NCI growth - Crowding Effect Steepness (D)*/
  protected ModelVector mp_fNCICrowdingEffectSteepness = new ModelVector(
      "NCI Crowding Effect Steepness (D)", "gr_nciCrowdingSteepness",
      "gr_ncstVal", 0, ModelVector.FLOAT);

  /**NCI growth - Neighbor storm effect - medium damage*/
  protected ModelVector mp_fNCINeighStormEffMed = new ModelVector(
      "NCI Neighbor Storm Damage (eta) - Medium (0-1)",
      "gr_nciNeighStormEffMedDmg", "gr_nnsemdVal", 0, ModelVector.FLOAT);

  /**NCI growth - Neighbor storm effect - full damage*/
  protected ModelVector mp_fNCINeighStormEffFull = new ModelVector(
      "NCI Neighbor Storm Damage (eta) - Complete (0-1)",
      "gr_nciNeighStormEffFullDmg", "gr_nnsefdVal", 0, ModelVector.FLOAT);

  /**NCI growth - Size effect mode (X0)*/
  protected ModelVector mp_fNCISizeEffectMode = new ModelVector(
      "NCI Size Effect Mode, in cm (X0)", "gr_nciSizeEffectMode",
      "gr_nsemVal", 0, ModelVector.FLOAT);

  /**NCI growth - Size effect variance (Xb)*/
  protected ModelVector mp_fNCISizeEffectVariance = new ModelVector(
      "NCI Size Effect Variance, in cm (Xb)", "gr_nciSizeEffectVariance",
      "gr_nsevVal", 0, ModelVector.FLOAT);

  /**NCI growth - Shading coefficient (m)*/
  protected ModelVector mp_fNCIShadingEffectCoefficient = new ModelVector(
      "NCI Shading Effect Coefficient (m)", "gr_nciShadingCoefficient",
      "gr_nscVal", 0, ModelVector.FLOAT);

  /**NCI growth - Shading exponent (n)*/
  protected ModelVector mp_fNCIShadingEffectExponent = new ModelVector(
      "NCI Shading Effect Exponent (n)", "gr_nciShadingExponent",
      "gr_nseVal", 0, ModelVector.FLOAT);

  /**NCI growth - Storm effect - medium damage*/
  protected ModelVector mp_fNCIStormEffectMed = new ModelVector(
      "NCI Damage Effect - Medium Storm Damage (0-1)", "gr_nciStormEffMedDmg",
      "gr_nsemdVal", 0, ModelVector.FLOAT);

  /**NCI growth - Storm effect - full damage*/
  protected ModelVector mp_fNCIStormEffectFull = new ModelVector(
      "NCI Damage Effect - Complete Storm Damage (0-1)",
      "gr_nciStormEffFullDmg", "gr_nsefdVal", 0, ModelVector.FLOAT);

  /**Length of last suppression factor for each species*/
  protected ModelVector mp_fLengthOfLastSuppressionFactor = new ModelVector(
      "Length of Last Suppression Factor", "gr_lengthLastSuppFactor",
      "gr_llsfVal", 0, ModelVector.FLOAT);

  /**Length of current release factor for each species*/
  protected ModelVector mp_fLengthOfCurrentReleaseFactor = new ModelVector(
      "Length of Current Release Factor", "gr_lengthCurrReleaseFactor",
      "gr_lcrfVal", 0, ModelVector.FLOAT);

  /**Logistic diameter growth - asymptotic growth at full light*/
  protected ModelVector mp_fLogisticDiamAsympGrowthFullLight = new ModelVector(
      "Logistic - Asymptotic Diam Growth - Full Light in mm/yr (a)",
      "gr_diamAsympGrowthAtFullLight", "gr_dagaflVal", 0, ModelVector.FLOAT);

  /**Logistic height growth - asymptotic growth at full light*/
  protected ModelVector mp_fLogisticHeightAsympGrowthFullLight = new
      ModelVector(
      "Logistic - Asymptotic Height Growth - Full Light in cm/yr (a)",
      "gr_heightAsympGrowthAtFullLight", "gr_hagaflVal", 0, ModelVector.FLOAT);

  /**Logistic diameter growth - shape parameter 1 - b*/
  protected ModelVector mp_fLogisticDiamShape1b = new ModelVector(
      "Logistic - Diam Shape Param 1 (b)", "gr_diamShape1b", "gr_ds1bVal",
      0, ModelVector.FLOAT);

  /**Logistic height growth - shape parameter 1 - b*/
  protected ModelVector mp_fLogisticHeightShape1b = new ModelVector(
      "Logistic - Height Shape Param 1 (b)", "gr_heightShape1b",
      "gr_hs1bVal", 0, ModelVector.FLOAT);

  /**Logistic diameter growth - shape parameter 2 - c*/
  protected ModelVector mp_fLogisticDiamShape2c = new ModelVector(
      "Logistic - Diam Shape Param 2 (c)", "gr_diamShape2c", "gr_ds2cVal",
      0, ModelVector.FLOAT);

  /**Logistic height growth - shape parameter 2 - c*/
  protected ModelVector mp_fLogisticHeightShape2c = new ModelVector(
      "Logistic - Height Shape Param 2 (c)", "gr_heightShape2c", "gr_hs2cVal",
      0, ModelVector.FLOAT);

  /**Simple linear diameter growth - intercept*/
  protected ModelVector mp_fSimpLinDiamIntercept = new ModelVector(
      "Simple Linear - Diam Intercept in mm/yr (a)",
      "gr_diamSimpleLinearIntercept",
      "gr_dsliVal", 0, ModelVector.FLOAT);

  /**Simple linear height growth - intercept*/
  protected ModelVector mp_fSimpLinHeightIntercept = new ModelVector(
      "Simple Linear - Height Intercept in cm/yr (a)",
      "gr_heightSimpleLinearIntercept",
      "gr_hsliVal", 0, ModelVector.FLOAT);

  /**Simple linear diameter growth - slope*/
  protected ModelVector mp_fSimpLinDiamSlope = new ModelVector(
      "Simple Linear - Diam Slope (b)", "gr_diamSimpleLinearSlope",
      "gr_dslsVal", 0, ModelVector.FLOAT);

  /**Simple linear height growth - slope*/
  protected ModelVector mp_fSimpLinHeightSlope = new ModelVector(
      "Simple Linear - Height Slope (b)", "gr_heightSimpleLinearSlope",
      "gr_hslsVal", 0, ModelVector.FLOAT);

  /** Linear growth with exponential reduction for shade diameter growth - intercept*/
  protected ModelVector mp_fLinShadeDiamIntercept = new ModelVector(
      "Shaded Linear - Diam Intercept in mm/yr (a)",
      "gr_diamShadedLinearIntercept",
      "gr_dshliVal", 0, ModelVector.FLOAT);

  /** Linear growth with exponential reduction for shade height growth - intercept*/
  protected ModelVector mp_fLinShadeHeightIntercept = new ModelVector(
      "Shaded Linear - Height Intercept in cm/yr (a)",
      "gr_heightShadedLinearIntercept",
      "gr_hshliVal", 0, ModelVector.FLOAT);

  /** Linear growth with exponential reduction for shade diameter growth - slope*/
  protected ModelVector mp_fLinShadeDiamSlope = new ModelVector(
      "Shaded Linear - Diam Slope (b)", "gr_diamShadedLinearSlope",
      "gr_dshlsVal", 0, ModelVector.FLOAT);

  /** Linear growth with exponential reduction for shade height growth - slope*/
  protected ModelVector mp_fLinShadeHeightSlope = new ModelVector(
      "Shaded Linear - Height Slope (b)", "gr_heightShadedLinearSlope",
      "gr_hshlsVal", 0, ModelVector.FLOAT);

  /** Linear growth with exponential reduction for shade diameter growth - shade exponent*/
  protected ModelVector mp_fLinShadeDiamShadeExp = new ModelVector(
      "Shaded Linear - Diam Shade Exponent (c)", "gr_diamShadedLinearShadeExp",
      "gr_dshlseVal", 0, ModelVector.FLOAT);

  /** Linear growth with exponential reduction for shade height growth - shade exponent*/
  protected ModelVector mp_fLinShadeHeightShadeExp = new ModelVector(
      "Shaded Linear - Height Shade Exponent (c)",
      "gr_heightShadedLinearShadeExp", "gr_hshlseVal", 0, ModelVector.FLOAT);

  /** Logistic growth with size dependent asymptote - diameter intercept*/
  protected ModelVector mp_fSizeDepLogisticDiamIntercept = new ModelVector(
      "Size Dep. Logistic - Diam Intercept (a)",
      "gr_diamSizeDepLogisticIntercept", "gr_dsdliVal", 0, ModelVector.FLOAT);

  /** Logistic growth with size dependent asymptote - height intercept*/
  protected ModelVector mp_fSizeDepLogisticHeightIntercept = new ModelVector(
      "Size Dep. Logistic - Height Intercept (a)",
      "gr_heightSizeDepLogisticIntercept", "gr_hsdliVal", 0, ModelVector.FLOAT);

  /** Logistic growth with size dependent asymptote - diameter slope*/
  protected ModelVector mp_fSizeDepLogisticDiamSlope = new ModelVector(
      "Size Dep. Logistic - Diam Slope (b)", "gr_diamSizeDepLogisticSlope",
      "gr_dsdlsVal", 0, ModelVector.FLOAT);

  /** Logistic growth with size dependent asymptote - height slope*/
  protected ModelVector mp_fSizeDepLogisticHeightSlope = new ModelVector(
      "Size Dep. Logistic - Height Slope (b)", "gr_heightSizeDepLogisticSlope",
      "gr_hsdlsVal", 0, ModelVector.FLOAT);

  /** Logistic growth with size dependent asymptote - diameter shape parameter 1 - c*/
  protected ModelVector mp_fSizeDepLogisticDiamShape1c = new ModelVector(
      "Size Dep. Logistic - Diam Shape Param 1 (c)",
      "gr_diamSizeDepLogisticShape1c", "gr_dsdls1cVal", 0, ModelVector.FLOAT);

  /** Logistic growth with size dependent asymptote - height shape parameter 1 - c*/
  protected ModelVector mp_fSizeDepLogisticHeightShape1c = new ModelVector(
      "Size Dep. Logistic - Height Shape Param 1 (c)",
      "gr_heightSizeDepLogisticShape1c", "gr_hsdls1cVal", 0,
      ModelVector.FLOAT);

  /** Logistic growth with size dependent asymptote - diameter shape parameter 2 - d*/
  protected ModelVector mp_fSizeDepLogisticDiamShape2d = new ModelVector(
      "Size Dep. Logistic - Diam Shape Param 2 (d)",
      "gr_diamSizeDepLogisticShape2d", "gr_dsdls2dVal", 0, ModelVector.FLOAT);

  /** Logistic growth with size dependent asymptote - height shape parameter 2 - d*/
  protected ModelVector mp_fSizeDepLogisticHeightShape2d = new ModelVector(
      "Size Dep. Logistic - Height Shape Param 2 (d)",
      "gr_heightSizeDepLogisticShape2d", "gr_hsdls2dVal", 0, ModelVector.FLOAT);

  /** Lognormal diameter growth - growth increment at diameter 36*/
  protected ModelVector mp_fLognormalDiamIncAtDiam36 = new ModelVector(
      "Lognormal - Diam Growth Increment at Diam 36, in mm/yr (a)",
      "gr_diamLognormalIncAtDiam36", "gr_dliad36Val", 0, ModelVector.FLOAT);

  /** Lognormal height growth - height increment at diameter 36*/
  protected ModelVector mp_fLognormalHeightIncAtDiam36 = new ModelVector(
      "Lognormal - Height Growth Increment at Diam 36, in cm/yr (a)",
      "gr_heightLognormalIncAtDiam36", "gr_hliad36Val", 0, ModelVector.FLOAT);

  /** Lognormal diameter growth - shape parameter*/
  protected ModelVector mp_fLognormalDiamShapeParam = new ModelVector(
      "Lognormal - Diam Shape Parameter (b)", "gr_diamLognormalShapeParam",
      "gr_dlspVal", 0, ModelVector.FLOAT);

  /** Lognormal height growth - shape parameter*/
  protected ModelVector mp_fLognormalHeightShapeParam = new ModelVector(
      "Lognormal - Height Shape Parameter (b)", "gr_heightLognormalShapeParam",
      "gr_hlspVal", 0, ModelVector.FLOAT);

  /** Lognormal diameter growth - effect of shade*/
  protected ModelVector mp_fLognormalDiamEffectOfShade = new ModelVector(
      "Lognormal - Diam Effect of Shade (c)", "gr_diamLognormalEffectOfShade",
      "gr_dleosVal", 0, ModelVector.FLOAT);

  /** Lognormal height growth - effect of shade*/
  protected ModelVector mp_fLognormalHeightEffectOfShade = new ModelVector(
      "Lognormal - Height Effect of Shade (c)",
      "gr_heightLognormalEffectOfShade", "gr_hleosVal", 0, ModelVector.FLOAT);

  /** Double resource relative growth - influence of resource*/
  protected ModelVector mp_fDoubleResourceInfluence = new ModelVector(
      "Double resource - Influence of Resource (C)",
      "gr_diamDoubleMMResourceInfluence", "gr_ddmmriVal", 0, ModelVector.FLOAT);

  /** Linear bi-level growth - threshold for high-light growth*/
  protected ModelVector mp_fBiLevelHighLightThreshold = new ModelVector(
      "Linear Bi-Level - Threshold for High-Light Growth (0 - 100)",
      "gr_linearBilevHiLiteThreshold", "gr_lbhltVal", 0, ModelVector.FLOAT);

  /** Linear bi-level growth - slope for high-light growth*/
  protected ModelVector mp_fBiLevelHighLightSlope = new ModelVector(
      "Linear Bi-Level - Slope for High-Light Growth (b)",
      "gr_linearBilevHiLiteSlope", "gr_lbhlsVal", 0, ModelVector.FLOAT);

  /** Linear bi-level growth - intercept for high-light growth*/
  protected ModelVector mp_fBiLevelHighLightIntercept = new ModelVector(
      "Linear Bi-Level - Intercept for High-Light Growth (a)",
      "gr_linearBilevHiLiteIntercept", "gr_lbhliVal", 0, ModelVector.FLOAT);

  /** Linear bi-level growth - slope for low-light growth*/
  protected ModelVector mp_fBiLevelLowLightSlope = new ModelVector(
      "Linear Bi-Level - Slope for Low-Light Growth (b)",
      "gr_linearBilevLoLiteSlope", "gr_lbllsVal", 0, ModelVector.FLOAT);

  /** Linear bi-level growth - intercept for low-light growth*/
  protected ModelVector mp_fBiLevelLowLightIntercept = new ModelVector(
      "Linear Bi-Level - Intercept for Low-Light Growth (a)",
      "gr_linearBilevLoLiteIntercept", "gr_lblliVal", 0, ModelVector.FLOAT);

  /** Lognormal bi-level growth - X0 for low-light growth*/
  protected ModelVector mp_fLogBiLevelLoLiteX0 = new ModelVector(
      "Lognormal Bi-Level - X0 for Low-Light Growth",
      "gr_lognormalBilevLoLiteX0", "gr_lbllx0Val", 0, ModelVector.FLOAT);

  /** Lognormal bi-level growth - Xb for low-light growth*/
  protected ModelVector mp_fLogBiLevelLoLiteXb = new ModelVector(
      "Lognormal Bi-Level - Xb for Low-Light Growth",
      "gr_lognormalBilevLoLiteXb", "gr_lbllxbVal", 0, ModelVector.FLOAT);

  /** Lognormal bi-level growth - max growth for low-light growth*/
  protected ModelVector mp_fLogBiLevelLoLiteMaxGrwth = new ModelVector(
      "Lognormal Bi-Level - Max Growth in Low Light (m)",
      "gr_lognormalBilevLoLiteMaxGrowth", "gr_lbllmgVal", 0, ModelVector.FLOAT);

  /** Lognormal bi-level growth - X0 for high-light growth*/
  protected ModelVector mp_fLogBiLevelHiLiteX0 = new ModelVector(
      "Lognormal Bi-Level - X0 for High-Light Growth",
      "gr_lognormalBilevHiLiteX0", "gr_lbhlx0Val", 0, ModelVector.FLOAT);

  /** Lognormal bi-level growth - Xb for high-light growth*/
  protected ModelVector mp_fLogBiLevelHiLiteXb = new ModelVector(
      "Lognormal Bi-Level - Xb for High-Light Growth",
      "gr_lognormalBilevHiLiteXb", "gr_lbhlxbVal", 0, ModelVector.FLOAT);

  /** Lognormal bi-level growth - max growth for high-light growth*/
  protected ModelVector mp_fLogBiLevelHiLiteMaxGrwth = new ModelVector(
      "Lognormal Bi-Level - Max Growth in High Light (m)",
      "gr_lognormalBilevHiLiteMaxGrowth", "gr_lbhlmgVal", 0, ModelVector.FLOAT);

  /** Lognormal bi-level growth - threshold for high-light growth*/
  protected ModelVector mp_fLogBiLevelHighLightThreshold = new ModelVector(
      "Lognormal Bi-Level - Threshold for High-Light Growth (0 - 100)",
      "gr_lognormalBilevHiLiteThreshold", "gr_lobhltVal", 0, ModelVector.FLOAT);

  /** PR semi-stochastic growth - height threshold for stochastic growth*/
  protected ModelVector mp_fPRHiteThreshold = new ModelVector(
      "PR - Height Threshold for Stochastic Growth (m)",
      "gr_prStochHiteThreshold", "gr_pshtVal", 0, ModelVector.FLOAT);

  /** PR semi-stochastic growth - "a" for deterministic growth*/
  protected ModelVector mp_fPRDetermA = new ModelVector(
      "PR - \"a\" Parameter for Deterministic Growth",
      "gr_prStochDetermA", "gr_psdaVal", 0, ModelVector.FLOAT);

  /** PR semi-stochastic growth - "b" for deterministic growth*/
  protected ModelVector mp_fPRDetermB = new ModelVector(
      "PR - \"b\" Parameter for Deterministic Growth",
      "gr_prStochDetermB", "gr_psdbVal", 0, ModelVector.FLOAT);

  /** PR semi-stochastic growth - mean DBH for stochastic growth*/
  protected ModelVector mp_fPRMeanDBH = new ModelVector(
      "PR - Mean DBH (cm) for Stochastic Growth",
      "gr_prStochMeanDBH", "gr_psmdVal", 0, ModelVector.FLOAT);

  /** PR semi-stochastic growth - DBH standard deviation for stochastic growth*/
  protected ModelVector mp_fPRDBHStdDev = new ModelVector(
      "PR - DBH Standard Deviation for Stochastic Growth",
      "gr_prStochStdDev", "gr_pssdVal", 0, ModelVector.FLOAT);

  /** PR storm bi-level growth - threshold for high-light growth*/
  protected ModelVector mp_fPRStormLevelHighLightThreshold = new ModelVector(
      "PR Storm Bi-Level - Threshold for High-Light Growth (0 - 100)",
      "gr_prBilevStmGrwthHiLiteThreshold", "gr_pbsghltVal", 0, ModelVector.FLOAT);

  /** PR storm bi-level growth - high-light growth "a"*/
  protected ModelVector mp_fPRStormBiLevelHighLightA = new ModelVector(
      "PR Storm Bi-Level - High-Light \"a\"",
      "gr_prBilevStmGrwthHiLiteA", "gr_pbsghlaVal", 0, ModelVector.FLOAT);

  /** PR storm bi-level growth - high-light growth "b"*/
  protected ModelVector mp_fPRStormBiLevelHighLightB = new ModelVector(
      "PR Storm Bi-Level - High-Light \"b\"",
      "gr_prBilevStmGrwthHiLiteB", "gr_pbsghlbVal", 0, ModelVector.FLOAT);

  /** PR storm bi-level growth - slope for low-light growth*/
  protected ModelVector mp_fPRStormBiLevelLowLightSlope = new ModelVector(
      "PR Storm Bi-Level - Slope for Low-Light Growth (b)",
      "gr_prBilevStmGrwthLoLiteSlope", "gr_pbsgllsVal", 0, ModelVector.FLOAT);

  /** PR storm bi-level growth - intercept for low-light growth*/
  protected ModelVector mp_fPRStormBiLevelLowLightIntercept = new ModelVector(
      "PR Storm Bi-Level - Intercept for Low-Light Growth (a)",
      "gr_prBilevStmGrwthLoLiteIntercept", "gr_pbsglliVal", 0, ModelVector.FLOAT);

  /** Browsed relative growth - browsed asymptotic diameter growth*/
  protected ModelVector mp_fBrowsedAsymptoticDiameterGrowth = new ModelVector(
     "Browsed Asymptotic Diameter Growth (A)",
     "gr_browsedAsympDiameterGrowth", "gr_badgVal", 0, ModelVector.FLOAT);

  /** Browsed relative growth - browsed slope of growth response*/
  protected ModelVector mp_fBrowsedSlopeOfGrowthResponse = new ModelVector(
     "Browsed Slope of Growth Response (S)", "gr_browsedSlopeGrowthResponse",
     "gr_bsgrVal", 0,  ModelVector.FLOAT);

  /** Browsed relative growth - browsed diameter exponent*/
  protected ModelVector mp_fBrowsedRelGrowthDiamExp = new ModelVector(
      "Browsed Diameter Exponent",
      "gr_browsedRelGrowthDiamExp", "gr_brgdeVal", 0, ModelVector.FLOAT);
  
  /** Michaelis-Menton negative growth - alpha */
  protected ModelVector mp_fMMNegGrowthAlpha = new ModelVector(
      "Michaelis-Menton with Negative Growth - Alpha",
      "gr_mmNegGrowthAlpha", "gr_mmngaVal", 0, ModelVector.FLOAT);
  
  /** Michaelis-Menton negative growth - beta  */
  protected ModelVector mp_fMMNegGrowthBeta = new ModelVector(
      "Michaelis-Menton with Negative Growth - Beta",
      "gr_mmNegGrowthBeta", "gr_mmngbVal", 0, ModelVector.FLOAT);
  
  /** Michaelis-Menton negative growth - gamma */
  protected ModelVector mp_fMMNegGrowthGamma = new ModelVector(
      "Michaelis-Menton with Negative Growth - Gamma",
      "gr_mmNegGrowthGamma", "gr_mmnggVal", 0, ModelVector.FLOAT);
  
  /** Michaelis-Menton negative growth - phi */
  protected ModelVector mp_fMMNegGrowthPhi = new ModelVector(
      "Michaelis-Menton with Negative Growth - Phi",
      "gr_mmNegGrowthPhi", "gr_mmngpVal", 0, ModelVector.FLOAT);
  
  /** Michaelis-Menton with photoinhibition - alpha */
  protected ModelVector mp_fMMPhotGrowthAlpha = new ModelVector(
      "Michaelis-Menton with Photoinhibition - Alpha",
      "gr_mmPhotGrowthAlpha", "gr_mmpgaVal", 0, ModelVector.FLOAT);
  
  /** Michaelis-Menton with photoinhibition - beta  */
  protected ModelVector mp_fMMPhotGrowthBeta = new ModelVector(
      "Michaelis-Menton with Photoinhibition - Beta",
      "gr_mmPhotGrowthBeta", "gr_mmpgbVal", 0, ModelVector.FLOAT);
  
  /** Michaelis-Menton with photoinhibition - D */
  protected ModelVector mp_fMMPhotGrowthD = new ModelVector(
      "Michaelis-Menton with Photoinhibition - D",
      "gr_mmPhotGrowthD", "gr_mmpgdVal", 0, ModelVector.FLOAT);
  
  /** Michaelis-Menton with photoinhibition - phi */
  protected ModelVector mp_fMMPhotGrowthPhi = new ModelVector(
      "Michaelis-Menton with Photoinhibition - Phi",
      "gr_mmPhotGrowthPhi", "gr_mmpgpVal", 0, ModelVector.FLOAT);
  
  /** Power height growth - n */
  protected ModelVector mp_fPowerHeightGrowthN = new ModelVector(
      "Power Height Growth - n",
      "gr_powerHeightN", "gr_phnVal", 0, ModelVector.FLOAT);
  
  /** Power height growth - exponent */
  protected ModelVector mp_fPowerHeightGrowthExp = new ModelVector(
      "Power Height Growth - Exp",
      "gr_powerHeightExp", "gr_pheVal", 0, ModelVector.FLOAT);
  
  /**Lagged post harvest growth - Multiplier constant in Max potential growth term*/
  protected ModelVector mp_fLagMaxGrowthConstant = new ModelVector(
      "Post Harvest Growth - Max Growth Constant",
      "gr_lagMaxGrowthConstant", "gr_lmgcVal", 0, ModelVector.FLOAT);

  /**Lagged post harvest growth - Effect of DBH on max potential growth term*/
  protected ModelVector mp_fLagMaxGrowthDBHEffect = new ModelVector(
      "Post Harvest Growth - DBH Growth Effect",
      "gr_lagMaxGrowthDbhEffect", "gr_lmgdbheVal", 0, ModelVector.FLOAT);

  /**Lagged post harvest growth - Effect of NCI on growth*/
  protected ModelVector mp_fLagNciConstant = new ModelVector(
      "Post Harvest Growth - NCI Constant ",
      "gr_lagNciConstant", "gr_lncicVal", 0, ModelVector.FLOAT);

  /**Lagged post harvest growth - Effect of DBH on NCI term*/
  protected ModelVector mp_fLagNciDbhEffect = new ModelVector(
      "Post Harvest Growth - DBH NCI Effect",
      "gr_lagNciDbhEffect", "gr_lncidbheVal", 0, ModelVector.FLOAT);

  /**Lagged post harvest growth - Rate parameter for time since harvest lag effect*/
  protected ModelVector mp_fLagTimeSinceHarvestRateParam = new ModelVector(
      "Post Harvest Growth - Time Since Harvest Rate Param",
      "gr_lagTimeSinceHarvestRateParam", "gr_ltshrpVal", 0, ModelVector.FLOAT);

  /**Lagged post harvest growth - NCI radius*/
  protected ModelFloat m_fLagNCIRadius = new ModelFloat(0, 
      "Post Harvest Growth - NCI Distance (m)", "gr_lagNciDistanceRadius"); 

  /**Morality rate at suppression*/
  protected ModelFloat m_fMortalityRateAtSuppression = new ModelFloat(0,
      "Mortality Threshold for Suppression", "gr_mortRateAtSuppression");

  /**NCI - NCI DBH divisor*/
  protected ModelFloat m_fNCIDbhDivisor = new ModelFloat(1,
      "NCI DBH Divisor (q)", "gr_nciDbhDivisor");
  
  /**BA NCI - BA divisor*/
  protected ModelFloat m_fBANCIBADivisor = new ModelFloat(1,
      "Basal Area NCI - BA Divisor", "gr_banciBADivisor");

  /**Years exceeding threshold before a tree is suppressed*/
  protected ModelInt m_iYrsExceedingThresholdBeforeSuppressed = new ModelInt(0,
      "Years Exceeding Threshold Before a Tree is Suppressed",
      "gr_yrsExceedThresholdBeforeSupp");

  /**NCI - Whether or not to include snags in NCI calculations*/
  protected ModelEnum m_iIncludeSnagsInNCI =
      new ModelEnum(new int[] {0, 1},
                    new String[] {"false", "true"},
                    "Include Snags in NCI Calculations",
                    "gr_nciIncludeSnagsInNCI");
  
  /**BA NCI - Whether or not to use only larger neighbors*/
  protected ModelEnum m_iBANCIOnlyLargerNeighbors =
      new ModelEnum(new int[] {0, 1},
                    new String[] {"false", "true"},
                    "Basal Area NCI - Use Only Larger Neighbors",
                    "gr_banciOnlyLargerNeighbors");

  /**
   * Constructor.
   * @param oManager GUIManager object.
   * @throws ModelException Passed through from called functions.  Shouldn't
   * ever be thrown.
   */
  public GrowthBehaviors(GUIManager oManager) throws ModelException {
    super(oManager, "Growth", "growth");

    m_sHelpID = "data.growth_behaviors";

    //Set up growth behaviors
    mp_oChildBehaviors = new Behavior[53];
    int iIndex = -1;

    //Default the snags in NCI flag to false
    m_iIncludeSnagsInNCI.SetValue("false");

    //*****************
    //Absolute growth limited by basal area - auto-height
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("absba",
                                         "Absolute growth limited to basal area increment - diam with auto height",
                                         "absbagrowth",
                                         (float) 1);
    //Data
    mp_oChildBehaviors[iIndex].AddRequiredData(m_fMortalityRateAtSuppression);
    mp_oChildBehaviors[iIndex].AddRequiredData(
        m_iYrsExceedingThresholdBeforeSuppressed);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAdultConstantAreaInc);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAsymptoticDiameterGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSlopeOfGrowthResponse);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLengthOfLastSuppressionFactor);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLengthOfCurrentReleaseFactor);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember(
        "Years suppressed", "yls", DataMember.INTEGER));
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember(
        "Years released", "ylr", DataMember.INTEGER));
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Absolute growth limited by radial increment - auto-height
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("absradial",
                                         "Absolute growth limited to radial increment - diam with auto height",
                                         "absradialgrowth",
                                         (float) 1);
    //Data
    mp_oChildBehaviors[iIndex].AddRequiredData(m_fMortalityRateAtSuppression);
    mp_oChildBehaviors[iIndex].AddRequiredData(
        m_iYrsExceedingThresholdBeforeSuppressed);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAdultConstantRadialGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAsymptoticDiameterGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSlopeOfGrowthResponse);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLengthOfLastSuppressionFactor);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLengthOfCurrentReleaseFactor);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember(
        "Years suppressed", "yls", DataMember.INTEGER));
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember(
        "Years released", "ylr", DataMember.INTEGER));
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //BA NCI growth - auto-height
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("nciba",
                                         "Basal area NCI growth - diam with auto height",
                                         "ncibagrowth",
                                         (float) 1.0);

    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIMaxPotentialGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIMaxCrowdingRadius);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCISizeSensToNCI);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCICrowdingEffectSlope);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCICrowdingEffectSteepness);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIMinNeighborDBH);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCISizeEffectMode);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCISizeEffectVariance);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_fBANCIBADivisor);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_iBANCIOnlyLargerNeighbors);
    //Disallow for seedlings
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Browsed relative growth - auto-height
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("browsed relative growth",
                                         "Browsed relative growth - diam with auto height",
                                         "browsed relative growth",
                                         (float) 1.0);

    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAsymptoticDiameterGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSlopeOfGrowthResponse);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fRelGrowthDiamExp);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBrowsedAsymptoticDiameterGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBrowsedSlopeOfGrowthResponse);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBrowsedRelGrowthDiamExp);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));


    //*****************
    //Constant basal area increment - auto-height
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("constba",
                                         "Constant basal area growth - diam with auto height",
                                         "constbagrowth", (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAdultConstantAreaInc);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Constant radial increment - auto-height
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("constradial",
                                         "Constant radial growth - diam with auto height",
                                         "constradialgrowth", (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAdultConstantRadialGrowth);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));
    
    //*****************
    //Lagged post harvest growth - auto-height
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("lagged post harvest growth",
                                         "Lagged post harvest growth - diam with auto height",
                                         "lagged post-harvest growth",
                                         (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLagMaxGrowthConstant);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLagMaxGrowthDBHEffect);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLagNciConstant);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLagNciDbhEffect);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLagTimeSinceHarvestRateParam);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_fLagNCIRadius);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Pre-harvest growth", "PreHarvGr", DataMember.FLOAT));

    //*****************
    //Simple linear regression - auto-height
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("linear",
                                         "Linear growth - diam with auto height",
                                         "simple linear growth",
                                         (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSimpLinDiamIntercept);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSimpLinDiamSlope);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Linear growth with exponential reduction for shade - auto-height
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("shaded linear",
                                         "Linear growth w/ exponential shade reduction - diam with auto height",
                                         "shaded linear growth", (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLinShadeDiamIntercept);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLinShadeDiamSlope);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLinShadeDiamShadeExp);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Linear bi-level growth - auto-height
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("linear bi-level",
                                         "Linear bi-level growth - diam with auto height",
                                         "linear bilevel growth", (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBiLevelHighLightThreshold);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBiLevelLowLightSlope);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBiLevelLowLightIntercept);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBiLevelHighLightSlope);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBiLevelHighLightIntercept);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Logistic growth - auto-height
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("logistic growth",
                                         "Logistic growth - diam with auto height",
                                         "logistic growth", (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLogisticDiamAsympGrowthFullLight);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLogisticDiamShape1b);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLogisticDiamShape2c);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Logistic growth with size dependent asymptote - auto-height
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("size dependent logistic growth",
                                         "Logistic growth w/ size dependent asymptote - diam with auto height",
                                         "size dependent logistic growth",
                                         (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSizeDepLogisticDiamIntercept);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSizeDepLogisticDiamSlope);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSizeDepLogisticDiamShape1c);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSizeDepLogisticDiamShape2d);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Lognormal growth with exponential reduction for shade - auto-height
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("lognormal growth",
                                         "Lognormal with exponential shade reduction - diam with auto height",
                                         "lognormal growth",
                                         (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLognormalDiamIncAtDiam36);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLognormalDiamShapeParam);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLognormalDiamEffectOfShade);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //NCI growth - auto-height
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("nci",
                                         "NCI growth - diam with auto height",
                                         "ncigrowth",
                                         (float) 2.1);

    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIMaxPotentialGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIMaxCrowdingRadius);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIAlpha);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIBeta);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCISizeSensToNCI);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCICrowdingEffectSlope);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCICrowdingEffectSteepness);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCINeighStormEffMed);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCINeighStormEffFull);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_fNCIDbhDivisor);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_iIncludeSnagsInNCI);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIMinNeighborDBH);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCISizeEffectMode);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCISizeEffectVariance);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIShadingEffectCoefficient);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIShadingEffectExponent);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIStormEffectMed);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIStormEffectFull);
    //Disallow for seedlings
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Puerto Rico storm bi-level growth - auto-height
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("pr storm bi-level",
                                         "Puerto Rico storm bi-level growth - diam with auto height",
                                         "PR storm bilevel growth", (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPRStormLevelHighLightThreshold);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPRStormBiLevelLowLightSlope);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPRStormBiLevelLowLightIntercept);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPRStormBiLevelHighLightA);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPRStormBiLevelHighLightB);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));


    //*****************
    //Relative growth limited by basal area - auto-height
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("relba",
                                          "Relative growth limited to basal area increment - diam with auto height",
                                          "relbagrowth",
                                          (float) 1.2);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAdultConstantAreaInc);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAsymptoticDiameterGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSlopeOfGrowthResponse);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fRelGrowthDiamExp);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Relative growth limited by radial increment - auto-height
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("relradial",
                                          "Relative growth limited to radial increment - diam with auto height",
                                          "relradialgrowth",
                                          (float) 1.2);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAdultConstantRadialGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAsymptoticDiameterGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSlopeOfGrowthResponse);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fRelGrowthDiamExp);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Non-limited absolute growth - auto-height
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("absunlim",
                                          "Non-limited absolute growth - diam with auto height",
                                          "absunlimgrowth", (float) 1);
    //Data
    mp_oChildBehaviors[iIndex].AddRequiredData(m_fMortalityRateAtSuppression);
    mp_oChildBehaviors[iIndex].AddRequiredData(
        m_iYrsExceedingThresholdBeforeSuppressed);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAsymptoticDiameterGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSlopeOfGrowthResponse);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLengthOfLastSuppressionFactor);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLengthOfCurrentReleaseFactor);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember(
        "Years suppressed", "yls", DataMember.INTEGER));
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember(
        "Years released", "ylr", DataMember.INTEGER));
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Non-limited relative growth - auto-height
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("relunlim",
                                          "Non-limited relative growth - diam with auto height",
                                          "relunlimgrowth", (float) 1.2);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAsymptoticDiameterGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSlopeOfGrowthResponse);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fRelGrowthDiamExp);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Double resource Michaelis-Menton relative growth - auto-height
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("double resource",
                                          "Double resource relative growth - diam with auto height",
                                          "Double resource relative diam with auto height", (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAsymptoticDiameterGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSlopeOfGrowthResponse);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fDoubleResourceInfluence);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Stochastic gap growth behavior
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("stochastic gap",
                                          "Stochastic Gap Growth",
                                          "Stochastic Gap Growth", (float) 1);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Absolute growth limited by basal area - diam only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("absba diam only",
                                          "Absolute growth limited to basal area increment - diam only",
                                          "absbagrowth diam only",
                                          (float) 1);
    //Data
    mp_oChildBehaviors[iIndex].AddRequiredData(m_fMortalityRateAtSuppression);
    mp_oChildBehaviors[iIndex].AddRequiredData(
        m_iYrsExceedingThresholdBeforeSuppressed);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAdultConstantAreaInc);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAsymptoticDiameterGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSlopeOfGrowthResponse);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLengthOfLastSuppressionFactor);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLengthOfCurrentReleaseFactor);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember(
        "Years suppressed", "yls", DataMember.INTEGER));
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember(
        "Years released", "ylr", DataMember.INTEGER));
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Absolute growth limited by radial increment - diam only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("absradial diam only",
                                          "Absolute growth limited to radial increment - diam only",
                                          "absradialgrowth diam only",
                                          (float) 1);
    //Data
    mp_oChildBehaviors[iIndex].AddRequiredData(m_fMortalityRateAtSuppression);
    mp_oChildBehaviors[iIndex].AddRequiredData(
        m_iYrsExceedingThresholdBeforeSuppressed);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAdultConstantRadialGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAsymptoticDiameterGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSlopeOfGrowthResponse);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLengthOfLastSuppressionFactor);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLengthOfCurrentReleaseFactor);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember(
        "Years suppressed", "yls", DataMember.INTEGER));
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember(
        "Years released", "ylr", DataMember.INTEGER));
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Diameter incrementer - diam only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("diameter incrementer",
                                 "Allometric diameter growth - diam only",
                                          "diameter incrementer",
                                          (float) 1);
    
    //*****************
    //BA NCI growth - diam only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("nciba diam only",
                                        "Basal area NCI growth - diam only",
                                        "ncibagrowth diam only",
                                        (float) 1.0);

    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIMaxPotentialGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIMaxCrowdingRadius);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCISizeSensToNCI);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCICrowdingEffectSlope);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCICrowdingEffectSteepness);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIMinNeighborDBH);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCISizeEffectMode);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCISizeEffectVariance);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_fBANCIBADivisor);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_iBANCIOnlyLargerNeighbors);
    //Disallow for seedlings
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Browsed relative growth - diam only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("browsed relative growth diam only",
                                         "Browsed relative growth - diam only",
                                         "browsed relative growth diam only",
                                         (float) 1.0);

    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAsymptoticDiameterGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSlopeOfGrowthResponse);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fRelGrowthDiamExp);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBrowsedAsymptoticDiameterGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBrowsedSlopeOfGrowthResponse);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBrowsedRelGrowthDiamExp);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));



    //*****************
    //Constant basal area increment - diam only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("constba diam only",
                                          "Constant basal area growth - diam only",
                                          "constbagrowth diam only", (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAdultConstantAreaInc);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Constant radial increment - diam only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("constradial diam only",
                                          "Constant radial growth - diam only",
                                          "constradialgrowth diam only",
                                          (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAdultConstantRadialGrowth);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Lagged post harvest growth - diam only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("lagged post harvest growth diam only",
                                         "Lagged post harvest growth - diam only",
                                         "lagged post-harvest growth diam only",
                                         (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLagMaxGrowthConstant);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLagMaxGrowthDBHEffect);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLagNciConstant);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLagNciDbhEffect);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLagTimeSinceHarvestRateParam);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_fLagNCIRadius);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Pre-harvest growth", "PreHarvGr", DataMember.FLOAT));
    
    //*****************
    //Simple linear regression - diam only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("linear diam only",
                                          "Linear growth - diam only",
                                          "simple linear growth diam only",
                                          (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSimpLinDiamIntercept);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSimpLinDiamSlope);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Linear bi-level growth - diam only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("linear bi-level diam only",
                                         "Linear bi-level growth - diam only",
                                         "linear bilevel growth diam only", (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBiLevelHighLightThreshold);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBiLevelLowLightSlope);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBiLevelLowLightIntercept);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBiLevelHighLightSlope);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBiLevelHighLightIntercept);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Linear growth with exponential reduction for shade - diam only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("shaded linear diam only",
                                          "Linear growth w/ exponential shade reduction - diam only",
                                          "shaded linear growth diam only",
                                          (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLinShadeDiamIntercept);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLinShadeDiamSlope);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLinShadeDiamShadeExp);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Logistic growth - diam only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("logistic growth diam only",
                                          "Logistic growth - diam only",
                                          "logistic growth diam only",
                                          (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLogisticDiamAsympGrowthFullLight);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLogisticDiamShape1b);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLogisticDiamShape2c);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Logistic growth with size dependent asymptote - diam only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior(
        "size dependent logistic growth diam only",
        "Logistic growth w/ size dependent asymptote - diam only",
        "size dependent logistic growth diam only",
        (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSizeDepLogisticDiamIntercept);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSizeDepLogisticDiamSlope);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSizeDepLogisticDiamShape1c);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSizeDepLogisticDiamShape2d);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Lognormal growth with exponential reduction for shade - diam only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior(
        "lognormal growth diam only",
        "Lognormal with exponential shade reduction - diam only",
        "lognormal growth diam only",
        (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLognormalDiamIncAtDiam36);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLognormalDiamShapeParam);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLognormalDiamEffectOfShade);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //NCI growth - diam only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("nci diam only",
                                          "NCI growth - diam only",
                                          "ncigrowth diam only",
                                          (float) 2.1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIMaxPotentialGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIMaxCrowdingRadius);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIAlpha);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIBeta);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCISizeSensToNCI);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCICrowdingEffectSlope);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCICrowdingEffectSteepness);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCINeighStormEffMed);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCINeighStormEffFull);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_fNCIDbhDivisor);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_iIncludeSnagsInNCI);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIMinNeighborDBH);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCISizeEffectMode);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCISizeEffectVariance);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIShadingEffectCoefficient);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIShadingEffectExponent);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIStormEffectMed);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIStormEffectFull);
    //Disallow for seedlings
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Relative growth limited by basal area - diam only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("relba diam only",
                                          "Relative growth limited to basal area increment - diam only",
                                          "relbagrowth diam only",
                                          (float) 1.1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAdultConstantAreaInc);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAsymptoticDiameterGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSlopeOfGrowthResponse);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fRelGrowthDiamExp);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Relative growth limited by radial increment - diam only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("relradial diam only",
                                          "Relative growth limited to radial increment - diam only",
                                          "relradialgrowth diam only",
                                          (float) 1.1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAdultConstantRadialGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAsymptoticDiameterGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSlopeOfGrowthResponse);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fRelGrowthDiamExp);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Non-limited absolute growth - diam only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("absunlim diam only",
                                          "Non-limited absolute growth - diam only",
                                          "absunlimgrowth diam only", (float) 1);
    //Data
    mp_oChildBehaviors[iIndex].AddRequiredData(m_fMortalityRateAtSuppression);
    mp_oChildBehaviors[iIndex].AddRequiredData(
        m_iYrsExceedingThresholdBeforeSuppressed);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAsymptoticDiameterGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSlopeOfGrowthResponse);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLengthOfLastSuppressionFactor);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLengthOfCurrentReleaseFactor);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember(
        "Years suppressed", "yls", DataMember.INTEGER));
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember(
        "Years released", "ylr", DataMember.INTEGER));
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Non-limited relative growth - diam only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("relunlim diam only",
                                          "Non-limited relative growth - diam only",
                                          "relunlimgrowth diam only", (float) 1.1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAsymptoticDiameterGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSlopeOfGrowthResponse);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fRelGrowthDiamExp);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Puerto Rico semi-stochastic growth - diam only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("pr semi-stochastic diam only",
                                          "Puerto Rico semi-stochastic - diam only",
                                          "PR semi-stochastic diam only", (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPRHiteThreshold);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPRDetermA);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPRDetermB);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPRMeanDBH);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPRDBHStdDev);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Double resource Michaelis-Menton relative growth - auto-height
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("double resource diam only",
                                          "Double resource relative growth - diam only",
                                          "Double resource relative diam only", (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAsymptoticDiameterGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSlopeOfGrowthResponse);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fDoubleResourceInfluence);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Diameter growth", "Growth", DataMember.FLOAT));

    //*****************
    //Allometric height growth
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("height incrementer",
                                          "Allometric height growth",
                                          "height incrementer",
                                          (float) 1);

    //*****************
    //Simple linear regression - height only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("linear height only",
                                          "Linear growth - height only",
                                          "simple linear growth height only",
                                          (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSimpLinHeightIntercept);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSimpLinHeightSlope);

    //*****************
    //Linear growth with exponential reduction for shade - height only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("shaded linear height only",
                                          "Linear growth w/ exponential shade reduction - height only",
                                          "shaded linear growth height only",
                                          (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLinShadeHeightIntercept);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLinShadeHeightSlope);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLinShadeHeightShadeExp);

    //*****************
    //Logistic growth - height only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("logistic growth height only",
                                          "Logistic growth - height only",
                                          "logistic growth height only",
                                          (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(
        mp_fLogisticHeightAsympGrowthFullLight);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLogisticHeightShape1b);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLogisticHeightShape2c);

    //*****************
    //Logistic growth with size dependent asymptote - height only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior(
        "size dependent logistic growth height only",
        "Logistic growth w/ size dependent asymptote - height only",
        "size dependent logistic growth height only",
        (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSizeDepLogisticHeightIntercept);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSizeDepLogisticHeightSlope);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSizeDepLogisticHeightShape1c);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSizeDepLogisticHeightShape2d);

    //*****************
    //Lognormal growth with exponential reduction for shade - height only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior(
        "lognormal growth height only",
        "Lognormal with exponential shade reduction - height only",
        "lognormal growth height only",
        (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLognormalHeightIncAtDiam36);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLognormalHeightShapeParam);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLognormalHeightEffectOfShade);

    //*****************
    //Lognormal bi-level growth - height only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior(
        "lognormal bi-level growth height only",
        "Lognormal bi-level - height only",
        "log bilevel growth height only",
        (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLogBiLevelHighLightThreshold);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLogBiLevelLoLiteMaxGrwth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLogBiLevelLoLiteX0);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLogBiLevelLoLiteXb);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLogBiLevelHiLiteMaxGrwth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLogBiLevelHiLiteX0);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLogBiLevelHiLiteXb);
    
    //*****************
    //Michaelis Menton with negative growth - height only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior(
        "Michaelis Menton with negative growth height only",
        "Michaelis Menton with negative growth - height only",
        "Michaelis Menten negative growth height only",
        (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMMNegGrowthAlpha);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMMNegGrowthBeta);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMMNegGrowthGamma);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMMNegGrowthPhi);
    
    //*****************
    //Michaelis Menton with photoinhibition - height only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior(
        "Michaelis Menton with photoinhibition height only",
        "Michaelis Menton with photoinhibition - height only",
        "Michaelis Menten photoinhibition growth height only",
        (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMMPhotGrowthAlpha);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMMPhotGrowthBeta);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMMPhotGrowthD);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMMPhotGrowthPhi);
    
    //*****************
    //Power function growth - height only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior(
        "Power growth - height only",
        "Power growth - height only",
        "power growth height only",
        (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPowerHeightGrowthN);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPowerHeightGrowthExp);
    
    //*****************
    //Relative growth - height only
    //*****************
    iIndex++;
    mp_oChildBehaviors[iIndex] = new Behavior("relative height",
                                     "Relative growth - height only",
                                      "relative michaelis-menton height growth", 
                                      (float) 1.2);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAsymptoticHeightGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSlopeOfHeightGrowthResponse);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fRelGrowthHeightExp);



    mp_oAllData.add(mp_fNCIMaxPotentialGrowth);
    mp_oAllData.add(mp_fNCIMaxCrowdingRadius);
    mp_oAllData.add(mp_fNCIAlpha);
    mp_oAllData.add(mp_fNCIBeta);
    mp_oAllData.add(mp_fNCISizeSensToNCI);
    mp_oAllData.add(mp_fNCICrowdingEffectSlope);
    mp_oAllData.add(mp_fNCICrowdingEffectSteepness);
    mp_oAllData.add(mp_fNCINeighStormEffMed);
    mp_oAllData.add(mp_fNCINeighStormEffFull);
    mp_oAllData.add(m_fNCIDbhDivisor);
    mp_oAllData.add(m_iIncludeSnagsInNCI);
    mp_oAllData.add(mp_fNCIMinNeighborDBH);
    mp_oAllData.add(mp_fNCISizeEffectMode);
    mp_oAllData.add(mp_fNCISizeEffectVariance);
    mp_oAllData.add(mp_fNCIShadingEffectCoefficient);
    mp_oAllData.add(mp_fNCIShadingEffectExponent);
    mp_oAllData.add(mp_fNCIStormEffectMed);
    mp_oAllData.add(mp_fNCIStormEffectFull);
    mp_oAllData.add(mp_fAdultConstantRadialGrowth);
    mp_oAllData.add(mp_fAdultConstantAreaInc);
    mp_oAllData.add(mp_fAsymptoticDiameterGrowth);
    mp_oAllData.add(mp_fSlopeOfGrowthResponse);
    mp_oAllData.add(m_fMortalityRateAtSuppression);
    mp_oAllData.add(m_iYrsExceedingThresholdBeforeSuppressed);
    mp_oAllData.add(mp_fLengthOfLastSuppressionFactor);
    mp_oAllData.add(mp_fLengthOfCurrentReleaseFactor);
    mp_oAllData.add(mp_fLognormalDiamIncAtDiam36);
    mp_oAllData.add(mp_fLognormalDiamShapeParam);
    mp_oAllData.add(mp_fLognormalDiamEffectOfShade);
    mp_oAllData.add(mp_fLogisticDiamAsympGrowthFullLight);
    mp_oAllData.add(mp_fLogisticDiamShape1b);
    mp_oAllData.add(mp_fLogisticDiamShape2c);
    mp_oAllData.add(mp_fSizeDepLogisticDiamIntercept);
    mp_oAllData.add(mp_fSizeDepLogisticDiamSlope);
    mp_oAllData.add(mp_fSizeDepLogisticDiamShape1c);
    mp_oAllData.add(mp_fSizeDepLogisticDiamShape2d);
    mp_oAllData.add(mp_fLinShadeDiamIntercept);
    mp_oAllData.add(mp_fLinShadeDiamSlope);
    mp_oAllData.add(mp_fLinShadeDiamShadeExp);
    mp_oAllData.add(mp_fSimpLinDiamIntercept);
    mp_oAllData.add(mp_fSimpLinDiamSlope);
    mp_oAllData.add(mp_fLognormalHeightIncAtDiam36);
    mp_oAllData.add(mp_fLognormalHeightShapeParam);
    mp_oAllData.add(mp_fLognormalHeightEffectOfShade);
    mp_oAllData.add(mp_fLogisticHeightAsympGrowthFullLight);
    mp_oAllData.add(mp_fLogisticHeightShape1b);
    mp_oAllData.add(mp_fLogisticHeightShape2c);
    mp_oAllData.add(mp_fSizeDepLogisticHeightIntercept);
    mp_oAllData.add(mp_fSizeDepLogisticHeightSlope);
    mp_oAllData.add(mp_fSizeDepLogisticHeightShape1c);
    mp_oAllData.add(mp_fSizeDepLogisticHeightShape2d);
    mp_oAllData.add(mp_fLinShadeHeightIntercept);
    mp_oAllData.add(mp_fLinShadeHeightSlope);
    mp_oAllData.add(mp_fLinShadeHeightShadeExp);
    mp_oAllData.add(mp_fSimpLinHeightIntercept);
    mp_oAllData.add(mp_fSimpLinHeightSlope);
    mp_oAllData.add(mp_fDoubleResourceInfluence);
    mp_oAllData.add(mp_fBiLevelHighLightThreshold);
    mp_oAllData.add(mp_fBiLevelLowLightSlope);
    mp_oAllData.add(mp_fBiLevelLowLightIntercept);
    mp_oAllData.add(mp_fBiLevelHighLightSlope);
    mp_oAllData.add(mp_fBiLevelHighLightIntercept);
    mp_oAllData.add(mp_fLogBiLevelHighLightThreshold);
    mp_oAllData.add(mp_fLogBiLevelLoLiteMaxGrwth);
    mp_oAllData.add(mp_fLogBiLevelLoLiteX0);
    mp_oAllData.add(mp_fLogBiLevelLoLiteXb);
    mp_oAllData.add(mp_fLogBiLevelHiLiteMaxGrwth);
    mp_oAllData.add(mp_fLogBiLevelHiLiteX0);
    mp_oAllData.add(mp_fLogBiLevelHiLiteXb);
    mp_oAllData.add(mp_fPRHiteThreshold);
    mp_oAllData.add(mp_fPRDetermA);
    mp_oAllData.add(mp_fPRDetermB);
    mp_oAllData.add(mp_fPRMeanDBH);
    mp_oAllData.add(mp_fPRDBHStdDev);
    mp_oAllData.add(mp_fRelGrowthDiamExp);
    mp_oAllData.add(mp_fPRStormLevelHighLightThreshold);
    mp_oAllData.add(mp_fPRStormBiLevelLowLightSlope);
    mp_oAllData.add(mp_fPRStormBiLevelLowLightIntercept);
    mp_oAllData.add(mp_fPRStormBiLevelHighLightA);
    mp_oAllData.add(mp_fPRStormBiLevelHighLightB);
    mp_oAllData.add(mp_fBrowsedAsymptoticDiameterGrowth);
    mp_oAllData.add(mp_fBrowsedSlopeOfGrowthResponse);
    mp_oAllData.add(mp_fBrowsedRelGrowthDiamExp);
    mp_oAllData.add(mp_fAsymptoticHeightGrowth);
    mp_oAllData.add(mp_fSlopeOfHeightGrowthResponse);
    mp_oAllData.add(mp_fRelGrowthHeightExp);
    mp_oAllData.add(mp_fMMNegGrowthAlpha);
    mp_oAllData.add(mp_fMMNegGrowthBeta);
    mp_oAllData.add(mp_fMMNegGrowthGamma);
    mp_oAllData.add(mp_fMMNegGrowthPhi);
    mp_oAllData.add(mp_fMMPhotGrowthAlpha);
    mp_oAllData.add(mp_fMMPhotGrowthBeta);
    mp_oAllData.add(mp_fMMPhotGrowthD);
    mp_oAllData.add(mp_fMMPhotGrowthPhi);
    mp_oAllData.add(mp_fPowerHeightGrowthN);
    mp_oAllData.add(mp_fPowerHeightGrowthExp);
    mp_oAllData.add(m_fBANCIBADivisor);
    mp_oAllData.add(m_iBANCIOnlyLargerNeighbors);
    mp_oAllData.add(mp_fLagMaxGrowthConstant);
    mp_oAllData.add(mp_fLagMaxGrowthDBHEffect);
    mp_oAllData.add(mp_fLagNciConstant);
    mp_oAllData.add(mp_fLagNciDbhEffect);
    mp_oAllData.add(mp_fLagTimeSinceHarvestRateParam);
    mp_oAllData.add(m_fLagNCIRadius);


    //Grids
    //mp_oGrids = new Grid[1];
    Grid oNewGrid;

    //Resource grid
    String sGridName = "Resource";

    //Create the data member
    DataMember[] p_oDataMembers = new DataMember[1];

    p_oDataMembers[0] = new DataMember("Resource", "Resource", DataMember.FLOAT);
    oNewGrid = new Grid(sGridName, p_oDataMembers, null, 8, 8);

    //Assign to double-resource relative growth
    oNewGrid = m_oManager.AddGrid(oNewGrid);
    GetBehaviorByKey("double resource").AddGrid(oNewGrid);
    GetBehaviorByKey("double resource diam only").AddGrid(oNewGrid);
    
    //Years since last harvest grid 
    sGridName = "Years Since Last Harvest"; 
    p_oDataMembers = new DataMember[2];
    p_oDataMembers[0] = new DataMember("Time", "Time", DataMember.INTEGER);
    p_oDataMembers[1] = new DataMember("LastUpdated", "LastUpdated", DataMember.INTEGER);
    oNewGrid = new Grid(sGridName, p_oDataMembers, null, 8, 8);
    //Assign to post lagged harvest growth
    GetBehaviorByKey("lagged post harvest growth").AddGrid(oNewGrid);
    GetBehaviorByKey("lagged post harvest growth diam only").AddGrid(oNewGrid); 
    
    //Defaults for Basal Area NCI parameters
    m_iBANCIOnlyLargerNeighbors.SetValue("true");
    m_fBANCIBADivisor.SetValue(1000);
  }

  /**
   * Performs setup tasks.  This method adds the NCI lambdas as required data,
   * and causes all storm damage parameters to default to 1.
   * @param oPop Tree population
   * @throws ModelException Won't throw.
   */
  public void DoSetup(TreePopulation oPop) throws ModelException {
    Behavior oBeh1 = GetBehaviorByKey("nci"),
        oBeh2 = GetBehaviorByKey("nci diam only");
    int iNumSpecies = oPop.GetNumberOfSpecies(), i, j;
    boolean bAlreadyLambdas = false;

    //See if we already had lambdas
    for (i = 0; i < mp_oAllData.size(); i++) {
      ModelData oData = (ModelData) mp_oAllData.elementAt(i);
      String sKey = oData.GetDescriptor().toLowerCase();
      if (sKey.indexOf("lambda neighbor") > -1) {
        bAlreadyLambdas = true;
        break;
      }
    }

    if (bAlreadyLambdas == false) {

      for (i = 0; i < iNumSpecies; i++) {
        String sSpName = oPop.GetSpeciesNameFromCode(i);
        ModelVector oVector = new ModelVector(
            sSpName.replace('_', ' ') + " NCI Lambda Neighbors",
            "gr_nci" + sSpName + "NeighborLambda", "gr_nlVal",
            0, ModelVector.FLOAT);

        mp_oAllData.add(oVector);
        oBeh1.AddRequiredData(oVector);
        oBeh2.AddRequiredData(oVector);
      }
    }
    else {
      //We already had lambdas.  What we want to do is preserve any for species
      //that didn't change, because they may have values.

      //Create an array of the lambda arrays, placed in the new species
      //order
      ModelData[] p_oNCI = new ModelData[iNumSpecies];
      int iCode;

      for (i = 0; i < iNumSpecies; i++) {
        p_oNCI[i] = null;
      }

      //Get the existing lambdas and put them where they go in our arrays -
      //any for species that were removed will be left behind
      for (i = 0; i < mp_oAllData.size(); i++) {
        ModelData oData = (ModelData) mp_oAllData.elementAt(i);
        String sKey = oData.GetDescriptor().toLowerCase();
        if (sKey.indexOf("lambda neighbor") > -1) {
          //get the species
          String sSpecies;
          sKey = oData.GetDescriptor();
          sSpecies = sKey.substring(0, sKey.indexOf(" NCI")).replace(' ', '_');
          iCode = oPop.GetSpeciesCodeFromName(sSpecies);
          if (iCode > -1) {
            p_oNCI[iCode] = oData;
          }
        }
      }

      //Now remove all lambdas from the required data and from all behaviors
      for (i = 0; i < mp_oAllData.size(); i++) {
        ModelData oData = (ModelData) mp_oAllData.elementAt(i);
        String sKey = oData.GetDescriptor().toLowerCase();
        if (sKey.indexOf("lambda neighbor") > -1) {
          mp_oAllData.remove(i);
          i--;
        }
      }
      for (i = 0; i < mp_oChildBehaviors.length; i++) {
        for (j = 0; j < mp_oChildBehaviors[i].mp_oRequiredData.size(); j++) {
          ModelData oData = (ModelData) mp_oChildBehaviors[i].
              mp_oRequiredData.elementAt(j);
          String sKey = oData.GetDescriptor().toLowerCase();
          if (sKey.indexOf("lambda neighbor") > -1) {
            mp_oChildBehaviors[i].mp_oRequiredData.remove(j);
            j--;
          }
        }
      }

      //Now add back the lambdas, creating new ones for any new species
      for (i = 0; i < p_oNCI.length; i++) {
        if (p_oNCI[i] == null) {
          String sSpName = oPop.GetSpeciesNameFromCode(i);
          p_oNCI[i] = new ModelVector(
              sSpName.replace('_', ' ') + " NCI Lambda Neighbors",
              "gr_nci" + sSpName + "NeighborLambda", "gr_nlVal",
              0, ModelVector.FLOAT);
        }
        mp_oAllData.add(p_oNCI[i]);
        oBeh1.AddRequiredData(p_oNCI[i]);
        oBeh2.AddRequiredData(p_oNCI[i]);
      }
    }

    //If the damage parameters have not already been set up, set all their
    //values to a default of 1.  Otherwise leave them alone.
    if (mp_fNCINeighStormEffMed.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_fNCINeighStormEffMed.GetValue().add(new Float(1.0));
      }
    }

    if (mp_fNCINeighStormEffFull.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_fNCINeighStormEffFull.GetValue().add(new Float(1.0));
      }
    }

    if (mp_fNCIStormEffectMed.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_fNCIStormEffectMed.GetValue().add(new Float(1.0));
      }
    }

    if (mp_fNCIStormEffectFull.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_fNCIStormEffectFull.GetValue().add(new Float(1.0));
      }
    }

    //Now set minimum adult DBH to 0 to allow easy backwards compatibility -
    //remove this section when you remove the deprecated m_iRelativizeNCI
    //LEM 03-16-05
    if (mp_fNCIMinNeighborDBH.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_fNCIMinNeighborDBH.GetValue().add(new Float(0.0));
      }
    }

    //Default the relative growth diameter exponents to 1
    //LEM 10-27-05
    if (mp_fRelGrowthDiamExp.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_fRelGrowthDiamExp.GetValue().add(new Float(1.0));
      }
    }
  }

  /**
   * Validates the data.
   * @param oPop TreePopulation object.
   * @throws ModelException if:
   * <ul>
   * <li>NCI Max. radius of neighbor effects is <= 0 for any species</li>
   * <li>NCI Max growth is <= 0 for any species</li>
   * <li>NCI size effect mode for any species is <= 0</li>
   * <li>NCI Size effect variance = 0 for any species</li>
   * <li>NCI Eta for any species for any damage category is not between 0 and
   * 1</li>
   * <li>NCI Storm Effect parameters for any species for any damage category
   * are not between 0 and 1</li>
   * <li>NCI DBH divisor is <= 0</li>
   * <li>Minimun neighbor DBH value is < 0</li>
   * <li>Linear bi-level growth is enabled and the light threshold is not
   * between 0 and 100</li>
   * <li>Lognormal bi-level growth is enabled and:
   * <ul>
   * <li>Max growth for high or low light is negative</li>
   * <li>X0 for high or low light is zero</li>
   * <li>Xb for high or low light is zero</li>
   * <li>Light threshold is not between 0 and 100</li>
   * </ul></li>
   * <li>PR storm bi-level growth is enabled and:
   * <ul>
   * <li>the high-light growth threshold is not between 0 and 100</li>
   * <li>the storm behavior is not enabled</li>
   * <li>the storm light behavior not enabled</li>
   * </ul></li>
   * </ul>
   */
  public void ValidateData(TreePopulation oPop) throws ModelException {
    ModelVector oVectorTest;
    Behavior oNCI;
    boolean[] p_bAppliesTo;
    int iNumSpecies = oPop.GetNumberOfSpecies(),
        i;

    //If no behaviors are enabled, exit
    if (!AnyBehaviorsEnabled()) {
      return;
    }

    p_bAppliesTo = new boolean[iNumSpecies];
    for (i = 0; i < p_bAppliesTo.length; i++) {
      p_bAppliesTo[i] = false;
    }

    if (IsThisDataUsed(mp_fAdultConstantRadialGrowth)) {
      MakeSureAllPositive(mp_fAdultConstantRadialGrowth, p_bAppliesTo);
    }

    //If an absolute growth behavior is enabled, validate suppression/release
    for (i = 0; i < mp_oChildBehaviors.length; i++) {
      if (mp_oChildBehaviors[i].GetKey().indexOf("abs") >= 0 &&
          mp_oChildBehaviors[i].m_bIsEnabled) {
        MakeSureRightSize(mp_fLengthOfLastSuppressionFactor, iNumSpecies);
        MakeSureRightSize(mp_fLengthOfCurrentReleaseFactor, iNumSpecies);
        MakeSureIsProportion(m_fMortalityRateAtSuppression);
        MakeSureGreaterThan(m_fMortalityRateAtSuppression, 0);
        MakeSureGreaterThan(m_iYrsExceedingThresholdBeforeSuppressed, 0);

        //Make sure that BC mortality is enabled
        Behavior oBeh = m_oManager.GetMortalityBehaviors().GetBehaviorByKey(
            "bcmort");
        if (oBeh == null) {
          throw (new ModelException(ErrorGUI.DATA_MISSING, "JAVA", "The " +
                                    "behavior \"" +
                                    mp_oChildBehaviors[i].GetDescriptor() +
                                    "\" requires the BC Mortality behavior."));
        }
        else if (oBeh.m_bIsEnabled == false) {
          throw (new ModelException(ErrorGUI.DATA_MISSING, "JAVA", "The " +
                                    "behavior \"" +
                                    mp_oChildBehaviors[i].GetDescriptor() +
                                    "\" requires the behavior \"" +
                                    oBeh.GetDescriptor() + "\"."));
        }
        break;
      }
    }

    //Validate that the NCI vectors are the right size if NCI growth is being
    //used
    if (GetBehaviorByKey("nci").m_bIsEnabled ||
        GetBehaviorByKey("nci diam only").m_bIsEnabled) {

      oNCI = GetBehaviorByKey("nci");
      p_bAppliesTo = oNCI.GetWhichSpeciesUsed(oPop);
      oNCI = GetBehaviorByKey("nci diam only");
      p_bAppliesTo = MergeBooleans(p_bAppliesTo, oNCI.GetWhichSpeciesUsed(oPop));

      //Count the number of lambdas
      int iNumLambdas = 0;
      for (i = 0; i < oNCI.GetNumberOfRequiredDataObjects(); i++) {
        ModelData oData = (ModelData) oNCI.GetRequiredData(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("lambda") > -1 &&
            sDescriptor.indexOf("pr nci") == -1) {
          oVectorTest = (ModelVector) oData;
          MakeSureAllAreProportions(oVectorTest, p_bAppliesTo);
          iNumLambdas++;
        }
      }

      if (iNumSpecies > iNumLambdas) {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
        "There is the wrong number of NCI lambda neighbors."));
      }
      MakeSureAllNonNegative(mp_fNCIMaxCrowdingRadius, p_bAppliesTo);
      MakeSureAllPositive(mp_fNCIMaxPotentialGrowth, p_bAppliesTo);
      MakeSureAllPositive(mp_fNCISizeEffectMode, p_bAppliesTo);
      MakeSureAllNonZero(mp_fNCISizeEffectVariance, p_bAppliesTo);
      MakeSureAllAreProportions(mp_fNCINeighStormEffFull, p_bAppliesTo);
      MakeSureAllAreProportions(mp_fNCINeighStormEffMed, p_bAppliesTo);
      MakeSureAllAreProportions(mp_fNCIStormEffectFull, p_bAppliesTo);
      MakeSureAllAreProportions(mp_fNCIStormEffectMed, p_bAppliesTo);
      MakeSureGreaterThan(m_fNCIDbhDivisor, 0);
      MakeSureAllNonNegative(mp_fNCIMinNeighborDBH); //this is for all species
    }

    //LEM 05-05-05
    if (GetBehaviorByKey("linear bi-level").m_bIsEnabled ||
        GetBehaviorByKey("linear bi-level diam only").m_bIsEnabled) {
      oNCI = GetBehaviorByKey("linear bi-level");
      p_bAppliesTo = oNCI.GetWhichSpeciesUsed(oPop);
      oNCI = GetBehaviorByKey("linear bi-level diam only");
      p_bAppliesTo = MergeBooleans(p_bAppliesTo, oNCI.GetWhichSpeciesUsed(oPop));
      MakeSureAllAreBounded(mp_fBiLevelHighLightThreshold, p_bAppliesTo, 0, 100);
    }

    //LEM 05-09-05
    if (GetBehaviorByKey("lognormal bi-level growth height only").m_bIsEnabled) {
      p_bAppliesTo = GetBehaviorByKey("lognormal bi-level growth height only").GetWhichSpeciesUsed(oPop);
      MakeSureAllAreBounded(mp_fLogBiLevelHighLightThreshold, p_bAppliesTo, 0, 100);
      MakeSureAllNonZero(mp_fLogBiLevelHiLiteX0, p_bAppliesTo);
      MakeSureAllNonZero(mp_fLogBiLevelLoLiteX0, p_bAppliesTo);
      MakeSureAllNonZero(mp_fLogBiLevelHiLiteXb, p_bAppliesTo);
      MakeSureAllNonZero(mp_fLogBiLevelLoLiteXb, p_bAppliesTo);
      MakeSureAllNonNegative(mp_fLogBiLevelHiLiteMaxGrwth, p_bAppliesTo);
      MakeSureAllNonNegative(mp_fLogBiLevelLoLiteMaxGrwth, p_bAppliesTo);
    }

    //LEM 02-21-06
    if (GetBehaviorByKey("pr storm bi-level").m_bIsEnabled) {
      p_bAppliesTo = GetBehaviorByKey("pr storm bi-level").GetWhichSpeciesUsed(oPop);
      MakeSureAllAreBounded(mp_fPRStormLevelHighLightThreshold, p_bAppliesTo, 0, 100);
      //Make sure storm behavior is enabled
      if (false == m_oManager.GetDisturbanceBehaviors().GetBehaviorByKey("Storms").m_bIsEnabled) {
        throw(new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA", "If the \""+
          GetBehaviorByKey("pr storm bi-level").GetDescriptor() +
          "\" behavior is used, the \"" +
          m_oManager.GetDisturbanceBehaviors().GetBehaviorByKey("Storms").GetDescriptor() +
          "\" behavior must also be used."));
      }
      //Make sure storm light behavior is enabled
      if (false == m_oManager.GetLightBehaviors().GetBehaviorByKey("storm light").m_bIsEnabled) {
        throw(new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA", "If the \""+
          GetBehaviorByKey("pr storm bi-level").GetDescriptor() +
          "\" behavior is used, the \"" +
          m_oManager.GetLightBehaviors().GetBehaviorByKey("storm light").GetDescriptor() +
          "\" behavior must also be used."));
      }
    }

    //Browsed relative growth validation
    if (GetBehaviorByKey("browsed relative growth").m_bIsEnabled ||
        GetBehaviorByKey("browsed relative growth diam only").m_bIsEnabled) {
      //For browsed relative growth, make sure that the browsed behavior is
      //enabled
      if (false == m_oManager.GetDisturbanceBehaviors().GetBehaviorByKey("Random Browse").m_bIsEnabled) {
        throw(new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA", "If the \""+
          GetBehaviorByKey("browsed relative growth").GetDescriptor() +
          "\" behavior is used, the \"" +
          m_oManager.GetDisturbanceBehaviors().GetBehaviorByKey("Random Browse").GetDescriptor() +
          "\" behavior must also be used."));
      }
    }
    
    //Michaelis-Menten with photoinhibition growth validation
    if (GetBehaviorByKey("Michaelis Menton with photoinhibition height only").m_bIsEnabled) {
      p_bAppliesTo = GetBehaviorByKey("Michaelis Menton with photoinhibition height only").GetWhichSpeciesUsed(oPop);
      MakeSureAllNonZero(mp_fMMPhotGrowthBeta, p_bAppliesTo); 
    }
    
    //Michaelis-Menten with negative growth validation
    if (GetBehaviorByKey("Michaelis Menton with negative growth height only").m_bIsEnabled) {
      p_bAppliesTo = GetBehaviorByKey("Michaelis Menton with negative growth height only").GetWhichSpeciesUsed(oPop);
      MakeSureAllNonZero(mp_fMMNegGrowthBeta, p_bAppliesTo); 
    }
  }

  /**
   * Before letting the base class write the XML, this verifies that there is
   * Resource grid information entered if the double resource relative behavior
   * is used.  I expect to remove this when there is some formal behavior for
   * taking care of the resources.
   * @param jOut FileWriter
   * @param oPop TreePopulation
   * @throws ModelException
   */
  public void WriteXML(java.io.BufferedWriter jOut, TreePopulation oPop) throws ModelException {
    if (GetBehaviorByKey("double resource").m_bIsEnabled ||
        GetBehaviorByKey("double resource diam only").m_bIsEnabled) {
      Grid oGrid = m_oManager.GetGridByName("Resource");
      if (oGrid.mp_oGridVals == null || oGrid.mp_oGridVals.size() == 0) {
        throw new ModelException(ErrorGUI.DATA_MISSING, "JAVA", "To use either the behavior \"" +
                                 GetBehaviorByKey("double resource").GetDescriptor() +
                                 "\"\n or the behavior \"" + GetBehaviorByKey("double resource diam only").GetDescriptor() +
                                 "\"\n you must populate grid values for the grid \"" +
                                 oGrid.GetName() + "\".");
      }
    }
    super.WriteXML(jOut, oPop);
  }

  /**
   * Updates the lambda neighbor names when a species name is changed.
   * @param sOldSpecies String Old name of the species, with underscores
   * instead of spaces (like the species names would come from the tree
   * population)
   * @param sNewSpecies String New name of the species, with underscores
   * instead of spaces (like the species names would come from the tree
   * population)
   */
  public void ChangeOfSpeciesName (String sOldSpecies, String sNewSpecies) {
    int i;

    sOldSpecies = sOldSpecies.replace('_', ' ');

    //Find the lambdas in our list
    for (i = 0; i < mp_oAllData.size(); i++) {
      ModelData oData = (ModelData) mp_oAllData.elementAt(i);
      String sKey = oData.GetDescriptor();
      if (sKey.indexOf(sOldSpecies) > -1) {
        ModelVector oVector = (ModelVector) oData;
        oVector.m_sDescriptor = sNewSpecies.replace('_', ' ') + " NCI Lambda Neighbors";
        oVector.m_sXMLTag = "gr_nci" + sNewSpecies + "NeighborLambda";
      }
    }
  }

  /**
   * Updates the lambda when a species is copied. The lambdas for that neighbor
   * are made identical to those being copied as well as entries for species
   * within the lambda.
   * @param iSpeciesCopyFrom int Species to copy.
   * @param iSpeciesCopyTo int Species that is the copy.
   * @throws ModelException if there is a problem.
   */
  public void CopySpecies (int iSpeciesCopyFrom, int iSpeciesCopyTo) throws ModelException {
    TreePopulation oPop = m_oManager.GetTreePopulation();
    ModelVector oCopyFrom = null, oCopyTo = null;
    ModelData oData;
    Float f1, f2;
    String sCopyFrom = oPop.GetSpeciesNameFromCode(iSpeciesCopyFrom).replace('_', ' '),
           sCopyTo = oPop.GetSpeciesNameFromCode(iSpeciesCopyTo).replace('_', ' ');
    int i;

    //Find the lambdas in our list
    for (i = 0; i < mp_oAllData.size(); i++) {
      oData = (ModelData) mp_oAllData.elementAt(i);
      String sKey = oData.GetDescriptor();
      if (sKey.indexOf(sCopyFrom) > -1) {
        oCopyFrom = (ModelVector) oData;
      }
      if (sKey.indexOf(sCopyTo) > -1) {
        oCopyTo = (ModelVector) oData;
      }
    }

    if (null == oCopyFrom || null == oCopyTo) {
      throw(new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA",
                               "Growth could not find NCI lambdas for neighbor"
                               + " species when copying species."));
    }

    //Do it differently depending on whether there are existing values or not
    //in the target array
    if (oCopyTo.GetValue().size() > 0) {
      for (i = 0; i < oCopyFrom.GetValue().size(); i++) {
        f1 = (Float)oCopyFrom.GetValue().elementAt(i);
        if (null == f1)
          f2 = null;
        else
          f2 = new Float(f1.floatValue());
        oCopyTo.GetValue().remove(i);
        oCopyTo.GetValue().add(i, f2);
      }
    } else {
    	for (i = 0; i < oCopyFrom.GetValue().size(); i++) {
        f1 = (Float)oCopyFrom.GetValue().elementAt(i);
        if (null == f1)
          f2 = null;
        else
          f2 = new Float(f1.floatValue());
        oCopyTo.GetValue().add(i, f2);
      }
    }

    super.CopySpecies(iSpeciesCopyFrom, iSpeciesCopyTo);
  }
}
