package javawrapper;

import junit.framework.*;
import java.io.FileWriter;
import java.io.File;

/**
 * Tests the SnagDynamicsBehaviors class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>March 8, 2007: Created (LEM)
 */
public class TestSnagDynamicsBehaviors extends TestCase {

  /**
   * Tests to make sure old substrate files are properly read.
   */
  public void testParFileRead() {
    GUIManager oManager = null;
    String sFileName = null;
    try {

      oManager = new GUIManager(null);

      //Load a file
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      SnagDynamicsBehaviors oSnags = oManager.GetSnagDynamicsBehaviors();

      //Verify that all the values are set correctly
      assertEquals(-0.805, oSnags.m_fSnagDecompTreeFallAlpha.GetValue(), 0.001);
      assertEquals(-0.016, oSnags.m_fSnagDecompTreeFallDelta.GetValue(), 0.001);
      assertEquals(-0.026, oSnags.m_fSnagDecompTreeFallTheta.GetValue(), 0.001);
      assertEquals(3.389, oSnags.m_fSnagDecompTreeFallIota.GetValue(), 0.001);
      assertEquals(-0.084, oSnags.m_fSnagDecompTreeFallLambda.GetValue(), 0.001);
      assertEquals(5.691, oSnags.m_fSnagDecompSnagFallAlpha.GetValue(), 0.001);
      assertEquals(0.177, oSnags.m_fSnagDecompSnagFallGamma2.GetValue(), 0.001);
      assertEquals(0.542, oSnags.m_fSnagDecompSnagFallGamma3.GetValue(), 0.001);
      assertEquals(0.702, oSnags.m_fSnagDecompSnagFallGamma4.GetValue(), 0.001);
      assertEquals(0.528, oSnags.m_fSnagDecompSnagFallGamma5.GetValue(), 0.001);
      assertEquals(-3.777, oSnags.m_fSnagDecompSnagFallZeta.GetValue(), 0.001);
      assertEquals(0.531, oSnags.m_fSnagDecompSnagFallEta.GetValue(), 0.001);
      assertEquals(0.157, oSnags.m_fSnagDecompSnagFallKappa.GetValue(), 0.001);
      assertEquals(0.290, oSnags.m_fSnagDecompLiveTo1Prob.GetValue(), 0.001);
      assertEquals(0.229, oSnags.m_fSnagDecompLiveTo2Prob.GetValue(), 0.001);
      assertEquals(0.196, oSnags.m_fSnagDecompLiveTo3Prob.GetValue(), 0.001);
      assertEquals(0.124, oSnags.m_fSnagDecompLiveTo4Prob.GetValue(), 0.001);
      assertEquals(0.161, oSnags.m_fSnagDecompLiveTo5Prob.GetValue(), 0.001);
      assertEquals(0.045, oSnags.m_fSnagDecomp1To1Prob.GetValue(), 0.001);
      assertEquals(0.186, oSnags.m_fSnagDecomp1To2Prob.GetValue(), 0.001);
      assertEquals(0.329, oSnags.m_fSnagDecomp1To3Prob.GetValue(), 0.001);
      assertEquals(0.166, oSnags.m_fSnagDecomp1To4Prob.GetValue(), 0.001);
      assertEquals(0.274, oSnags.m_fSnagDecomp1To5Prob.GetValue(), 0.001);
      assertEquals(0.165, oSnags.m_fSnagDecomp2To2Prob.GetValue(), 0.001);
      assertEquals(0.379, oSnags.m_fSnagDecomp2To3Prob.GetValue(), 0.001);
      assertEquals(0.204, oSnags.m_fSnagDecomp2To4Prob.GetValue(), 0.001);
      assertEquals(0.252, oSnags.m_fSnagDecomp2To5Prob.GetValue(), 0.001);
      assertEquals(0.351, oSnags.m_fSnagDecomp3To3Prob.GetValue(), 0.001);
      assertEquals(0.346, oSnags.m_fSnagDecomp3To4Prob.GetValue(), 0.001);
      assertEquals(0.303, oSnags.m_fSnagDecomp3To5Prob.GetValue(), 0.001);
      assertEquals(0.527, oSnags.m_fSnagDecomp4To4Prob.GetValue(), 0.001);
      assertEquals(0.473, oSnags.m_fSnagDecomp4To5Prob.GetValue(), 0.001);
      assertEquals(1, oSnags.m_fSnagDecomp5To5Prob.GetValue(), 0.001);
      
      assertEquals(6, oSnags.m_fSnagDecompMinSnagBreakHeight.GetValue(), 0.001);
      assertEquals(7, oSnags.m_fSnagDecompMaxSnagBreakHeight.GetValue(), 0.001);

      assertEquals( ( (Float) oSnags.mp_fSnagDecompTreeFallBeta.GetValue().elementAt(0)).floatValue(), 2, 0.001);
      assertEquals( ( (Float) oSnags.mp_fSnagDecompTreeFallBeta.GetValue().elementAt(1)).floatValue(), 0.057, 0.001);

      assertEquals( ( (Float) oSnags.mp_fSnagDecompSnagFallBeta.GetValue().elementAt(0)).floatValue(), 3, 0.001);
      assertEquals( ( (Float) oSnags.mp_fSnagDecompSnagFallBeta.GetValue().elementAt(1)).floatValue(), 0.092, 0.001);
    }
    catch (ModelException oErr) {
      fail("Snags dynamics parameter file read failed with message " +
           oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      if (sFileName != null) {
        new File(sFileName).delete();
      }
    }
  }

  /**
   * Tests ValidateData().
   * @throws ModelException if there's an uncaught validation problem.
   */
  public void testValidateData() throws ModelException {
    GUIManager oManager = null;
    String sFileName = null;
    SnagDynamicsBehaviors oSnags = null;
    try {

      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oSnags = oManager.GetSnagDynamicsBehaviors();
      oSnags.ValidateData(oManager.GetTreePopulation());
    }
    catch (ModelException oErr) {
      fail("Snag dynamics validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Case:  Live to 1 probability is not a proportion
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecompLiveTo1Prob.SetValue(-2);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch live to 1 probability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  Live to 2 probability is not a proportion
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecompLiveTo2Prob.SetValue(-2);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch live to 2 probability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  Live to 3 probability is not a proportion
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecompLiveTo3Prob.SetValue(-2);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch live to 3 probability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  Live to 4 probability is not a proportion
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecompLiveTo4Prob.SetValue(-2);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch live to 4 probability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  Live to 5 probability is not a proportion
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecompLiveTo5Prob.SetValue(-2);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch live to 5 probability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  1 to 1 probability is not a proportion
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecomp1To1Prob.SetValue(-2);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch 1 to 1 probability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  1 to 2 probability is not a proportion
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecomp1To2Prob.SetValue(-2);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch 1 to 2 probability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  1 to 3 probability is not a proportion
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecomp1To3Prob.SetValue(-2);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch 1 to 3 probability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  1 to 4 probability is not a proportion
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecomp1To4Prob.SetValue(-2);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch 1 to 4 probability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  1 to 5 probability is not a proportion
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecomp1To5Prob.SetValue(-2);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch 1 to 5 probability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  2 to 2 probability is not a proportion
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecomp2To2Prob.SetValue(-2);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch 2 to 2 probability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  2 to 3 probability is not a proportion
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecomp2To3Prob.SetValue(-2);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch 2 to 3 probability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  2 to 4 probability is not a proportion
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecomp2To4Prob.SetValue(-2);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch 2 to 4 probability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  2 to 5 probability is not a proportion
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecomp2To5Prob.SetValue(-2);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch 2 to 5 probability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  3 to 3 probability is not a proportion
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecomp3To3Prob.SetValue(-2);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch 3 to 3 probability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  3 to 4 probability is not a proportion
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecomp3To4Prob.SetValue(-2);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch 3 to 4 probability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  3 to 5 probability is not a proportion
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecomp3To5Prob.SetValue(-2);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch 3 to 5 probability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  4 to 4 probability is not a proportion
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecomp4To4Prob.SetValue(-2);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch 4 to 4 probability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  4 to 5 probability is not a proportion
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecomp4To5Prob.SetValue(-2);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch 4 to 5 probability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  5 to 5 probability is not a proportion
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecomp5To5Prob.SetValue(-2);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch 5 to 5 probability.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  Live transition probabilities don't add up to 1
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecompLiveTo5Prob.SetValue((float)0.9);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch live transition " +
           "probabilities not adding up to 1.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  Class 1 transition probabilities don't add up to 1
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecomp1To5Prob.SetValue((float)0.9);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch class 1 transition " +
           "probabilities not adding up to 1.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  Class 2 transition probabilities don't add up to 1
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecomp2To5Prob.SetValue((float)0.9);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch class 2 transition " +
           "probabilities not adding up to 1.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  Class 3 transition probabilities don't add up to 1
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecomp3To5Prob.SetValue((float)0.9);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch class 3 transition " +
           "probabilities not adding up to 1.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  Class 4 transition probabilities don't add up to 1
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecomp4To5Prob.SetValue((float)0.9);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch class 4 transition " +
           "probabilities not adding up to 1.");
    }
    catch (ModelException oErr) {
      ;
    }

    //Case:  Class 5 transition probabilities don't add up to 1
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecomp5To5Prob.SetValue((float)0.9);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch class 5 transition " +
           "probabilities not adding up to 1.");
    }
    catch (ModelException oErr) {
      ;
    }
    
    //Case: Max snag break height is not greater than min snag break height
    try {
      oManager.InputXMLParameterFile(sFileName);
      oSnags.m_fSnagDecompMinSnagBreakHeight.SetValue((float)10);
      oSnags.ValidateData(oManager.GetTreePopulation());
      fail("Snag dynamics testing didn't catch min snag break height greater" +
      		" than max snag break height.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println("Substrate ValidateData testing succeeded.");
  }

  /**
   * Writes a parameter file.
   * @throws IOException If there is a problem writing the file.
   * @return String Filename of file written.
   */
  protected String WriteXMLValidFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WritePlotAndTrees(oOut);

    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>snag decay class dynamics</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Snag\" />");
    oOut.write("<applyTo species=\"Species_2\" type=\"Snag\" />");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<snagDynamics>");
    oOut.write("<sd_snagDecompTreefallAlpha>-0.805</sd_snagDecompTreefallAlpha>");
    oOut.write("<sd_snagDecompTreefallBeta>");
    oOut.write("<sd_sdtbVal species=\"Species_1\">2</sd_sdtbVal>");
    oOut.write("<sd_sdtbVal species=\"Species_2\">0.057</sd_sdtbVal>");
    oOut.write("</sd_snagDecompTreefallBeta>");
    oOut.write("<sd_snagDecompTreefallDelta>-0.016</sd_snagDecompTreefallDelta>");
    oOut.write("<sd_snagDecompTreefallTheta>-0.026</sd_snagDecompTreefallTheta>");
    oOut.write("<sd_snagDecompTreefallIota>3.389</sd_snagDecompTreefallIota>");
    oOut.write("<sd_snagDecompTreefallLambda>-0.084</sd_snagDecompTreefallLambda>");
    oOut.write("<sd_snagDecompSnagfallAlpha>5.691</sd_snagDecompSnagfallAlpha>");
    oOut.write("<sd_snagDecompSnagfallBeta>");
    oOut.write("<sd_sdsbVal species=\"Species_1\">3</sd_sdsbVal>");
    oOut.write("<sd_sdsbVal species=\"Species_2\">0.092</sd_sdsbVal>");
    oOut.write("</sd_snagDecompSnagfallBeta>");
    oOut.write("<sd_snagDecompSnagfallGamma2>0.177</sd_snagDecompSnagfallGamma2>");
    oOut.write("<sd_snagDecompSnagfallGamma3>0.542</sd_snagDecompSnagfallGamma3>");
    oOut.write("<sd_snagDecompSnagfallGamma4>0.702</sd_snagDecompSnagfallGamma4>");
    oOut.write("<sd_snagDecompSnagfallGamma5>0.528</sd_snagDecompSnagfallGamma5>");
    oOut.write("<sd_snagDecompSnagfallZeta>-3.777</sd_snagDecompSnagfallZeta>");
    oOut.write("<sd_snagDecompSnagfallEta>0.531</sd_snagDecompSnagfallEta>");
    oOut.write("<sd_snagDecompSnagfallKappa>0.157</sd_snagDecompSnagfallKappa>");
    oOut.write("<sd_snagDecompLiveTo1Prob>0.290</sd_snagDecompLiveTo1Prob>");
    oOut.write("<sd_snagDecompLiveTo2Prob>0.229</sd_snagDecompLiveTo2Prob>");
    oOut.write("<sd_snagDecompLiveTo3Prob>0.196</sd_snagDecompLiveTo3Prob>");
    oOut.write("<sd_snagDecompLiveTo4Prob>0.124</sd_snagDecompLiveTo4Prob>");
    oOut.write("<sd_snagDecompLiveTo5Prob>0.161</sd_snagDecompLiveTo5Prob>");
    oOut.write("<sd_snagDecomp1To1Prob>0.045</sd_snagDecomp1To1Prob>");
    oOut.write("<sd_snagDecomp1To2Prob>0.186</sd_snagDecomp1To2Prob>");
    oOut.write("<sd_snagDecomp1To3Prob>0.329</sd_snagDecomp1To3Prob>");
    oOut.write("<sd_snagDecomp1To4Prob>0.166</sd_snagDecomp1To4Prob>");
    oOut.write("<sd_snagDecomp1To5Prob>0.274</sd_snagDecomp1To5Prob>");
    oOut.write("<sd_snagDecomp2To2Prob>0.165</sd_snagDecomp2To2Prob>");
    oOut.write("<sd_snagDecomp2To3Prob>0.379</sd_snagDecomp2To3Prob>");
    oOut.write("<sd_snagDecomp2To4Prob>0.204</sd_snagDecomp2To4Prob>");
    oOut.write("<sd_snagDecomp2To5Prob>0.252</sd_snagDecomp2To5Prob>");
    oOut.write("<sd_snagDecomp3To3Prob>0.351</sd_snagDecomp3To3Prob>");
    oOut.write("<sd_snagDecomp3To4Prob>0.346</sd_snagDecomp3To4Prob>");
    oOut.write("<sd_snagDecomp3To5Prob>0.303</sd_snagDecomp3To5Prob>");
    oOut.write("<sd_snagDecomp4To4Prob>0.527</sd_snagDecomp4To4Prob>");
    oOut.write("<sd_snagDecomp4To5Prob>0.473</sd_snagDecomp4To5Prob>");
    oOut.write("<sd_snagDecomp5To5Prob>1.000</sd_snagDecomp5To5Prob>");
    oOut.write("<sd_minSnagBreakHeight>6</sd_minSnagBreakHeight>");
    oOut.write("<sd_maxSnagBreakHeight>7</sd_maxSnagBreakHeight>");
    oOut.write("</snagDynamics>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * This writes plot and tree information.
   * @param oOut FileWriter
   * @throws IOException
   */
  private void WritePlotAndTrees(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>1</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("<tr_species speciesName=\"Species_2\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">39.48</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.0299</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</allometry>");
  }

}
